### Part 1.BRT model for mosquito model
library(dismo)
library(gbm)
library(pROC)
library(tidyverse)
library(Hmisc)
library(pheatmap)
library(RColorBrewer)
library(sf)
library(ggspatial)
library(PRROC)
library(factoextra)

setwd('')
var<-read.csv('var.csv') # independent variables
var_name<-read.csv('varname.csv')[,1] # variable names
case<-read.csv('mo_fenbu_study.csv') # mosquitoes data used to model
case_all <- read.csv('mo_fenbu_all.csv') # all mosquitoes data

###logistic model to calculate the weight of sampleing for reducing investigation error
case_var<- var[,c('ADCODE99',var_name)] %>%   
  dplyr::left_join(case,by=c('ADCODE99'))

case_logistic<- var[,c('ADCODE99',var_name)] %>%   
  dplyr::left_join(case_all[,c('ADCODE99','if_survey')],by=c('ADCODE99'))
aa <- glm(as.formula(str_c('if_survey~',str_c(var_name,collapse = '+'))),
          data=case_logistic,family = 'binomial')
bb <- step(aa,direction = 'backward')
dd <- glm(as.formula(str_c('if_survey~',
                           str_c(names(bb$coefficients)[-which(names(bb$coefficients)%in%c("(Intercept)",'animal_gt','bio3','pingyuan','pendi','animal_Ch'))],collapse = '+'))),
          data=case_logistic,family = 'binomial')
a <- predict(dd,newdata=case_logistic,type='response')
case_var$weight <- 1/a
case_var$weight <- case_var$weight/mean(case_var$weight[case_logistic$if_survey==1])

### built model
for(m_big_i in 1:26){
  ###m=targeted mosquitocase_var_model= data including all variables
  setwd('')  #each root to spefic mosquito
  m<-moi_study[m_big_i]
  id=which(names(case_var)=='ADCODE99')
  var_c=which(names(case_var) %in% var_name)
  var_mo=which(names(case_var)== m)
  # case_var$weight <- 1
  ir_inf<-10;ir_mo<-100
  
  data<-case_var[,c(var_mo,id,var_c,which(names(case_var)=='weight'))]
  names(data)[1]<-'case'
  var_model<-which(names(data) %in% var_name)
  
  data1<-data[data$case==1&!is.na(data$case),]
  data0<-data[data$case==0&!is.na(data$case),]
  n1<-ceiling(0.75*nrow(data1))
  n0<-ceiling(0.75*nrow(data0))
  dir.create(m)
  setwd(m)
  write.csv(rbind(data1,data0),'all.csv',row.names = F)
  write.csv(data,'all_NA.csv',row.names = F)
  for (i in 1:150){ #randomly split data for 150 times
    set.seed(1234+i)
    train1<-sample(1:nrow(data1),n1)
    set.seed(1234+i)
    train0<-sample(1:nrow(data0),n0)
    dftrain1<-data1[train1,];dftest1<-data1[-train1,];dftrain0<-data0[train0,];dftest0<-data0[-train0,]
    dftrain<-rbind(dftrain1,dftrain0);dftest<-rbind(dftest1,dftest0)
    dir.create('train');traini<-paste0('train\\train',i,'.csv')
    dir.create('test');testi<-paste0('test\\test',i,'.csv')
    write.csv(dftrain,traini,row.names = F);write.csv(dftest,testi,row.names = F)
  }
  ###step2.run models for 10 times with all variables to calculate relative influence
  model.all<-list();inf<-data.frame(var=c(names(data)[var_model]))
  while (ncol(inf)<(ir_inf+1)){
    i<-ncol(inf)
    train.data<-read.csv(paste0('train\\train',i,'.csv'))
    set.seed(1000+i)
    modeli<-gbm.step(data=train.data,gbm.x = var_model,gbm.y = 1,family = 'bernoulli', tree.complexity = 5,
                     site.weights =train.data$weight,
                     learning.rate = 0.005, bag.fraction = 0.75,n.folds = 10,max.trees = 3000,silent = T, plot.main = F)
    model.all[[i]]<-modeli
    infi<-summary(modeli);names(infi)<-c('var',paste0('inf',i))
    inf<-left_join(inf,infi,by='var')
    print(paste0(m,i))
  }
  inf <- inf %>%
    mutate(mean=apply(inf[,str_c('inf',1:10)],1,mean))
  
  var_inf<-function(inf,cut=2){
    aa<-as.character(inf[inf$mean>cut,1])
    a<-c(seq_along(data))[names(data) %in% aa]
    a
  }
  var_m<-var_inf(inf,cut = 2)
  
  ###step.3 run models for 100 times with filtered variables as final models
  model_z<-list();ROC<-list()
  i<-1;n_mo<-10;id_model_z<-1
  c_train<-c();c_test<-c()
  while(length(model_z)<(ir_mo+1)){
    n_traini<-paste0('train\\train',i+n_mo,'.csv');n_testi<-paste0('test\\test',i+n_mo,'.csv');
    train.data<-read.csv(n_traini);test.data<-read.csv(n_testi);all.data<-bind_rows(train.data,test.data)
    set.seed(12345+i)
    model_zi<-gbm.step(data<-train.data,gbm.x = var_m,gbm.y = 1,family = 'bernoulli', tree.complexity = 5,
                       # site.weights =train.data$weight,
                       learning.rate = 0.005, bag.fraction = 0.75,n.folds = 10,max.trees = 3000,silent = T, plot.main = F)
    print(str_c(i,'_100'))
    if(!is.null(model_zi)) {model_z[[id_model_z]]<-model_zi;id_model_z<-id_model_z+1
    c_train<-c(c_train,n_traini);c_test<-c(c_test,n_testi)}
    i<-i+1
  }
  pre_df<- function(name_df){
    df_re<-NULL
    for(i in 1:100){
      if(length(name_df)==100) data<-read.csv(name_df[i]) else data<-read.csv(name_df)
      codei<-str_c('code',i);orii<-str_c('ori',i);predi<- str_c('pre',i)
      preds_i<- predict.gbm(model_z[[i]],data,n.trees = model_z[[i]]$n.trees,type='response')
      if(is.null(df_re)) df_re<-cbind(data[,c(2,1)],preds_i) else df_re<-cbind(df_re,data[,c(2,1)],preds_i)
      names(df_re)[(i*3-2):(i*3)]<-c(codei,orii,predi)
    }
    df_re   
  }
  df_pred_train<-pre_df(c_train[1:100])
  df_pred_test<-pre_df(c_test[1:100])
  df_pred_all<-pre_df('all.csv')
  df_pred_allna<-pre_df('all_NA.csv')
  
  
  roc_wt<-function(df_pred){
    roc_l<-list();auc<-c();cut<-c()
    for(i in 1:100){
      roc_l[[i]]<-roc(df_pred[,(i*3-1)],df_pred[,(i*3)])
      auc<-c(auc,roc_l[[i]]$auc)
      cut<-c(cut,coords(roc_l[[i]],'best',transpose=T)['threshold'])
    }
    roc_l[[101]]<- auc;roc_l[[102]]<- cut
    roc_l
  }
  roc_train<-roc_wt(df_pred_train);roc_test<-roc_wt(df_pred_test);roc_all<-roc_wt(df_pred_all)
  
  
  df_roc<-data.frame(name=c('train','test','all'),
                     auc=c(mean(roc_train[[101]]),mean(roc_test[[101]]),mean(roc_all[[101]])),
                     cut=c(mean(roc_train[[102]]),mean(roc_test[[102]]),mean(roc_all[[102]])))
  
  
  df_pred_allna$mean<-apply(df_pred_allna[,(1:100)*3],1,mean)
  df_pred_allna$if_not<-ifelse(df_pred_allna$mean>mean(roc_all[[102]],na.rm = T),1,0)
  if_ci<-function(a){
    a_n<-c()
    for(i in 1:100){
      if(is.na(roc_all[[102]][i])) ai<-NA else {if(a[i*3]>roc_all[[102]][i]) ai<-1 else ai<-0}
      a_n<-c(a_n,ai)
    }
    sum(a_n,na.rm = T)/sum(!is.na(a_n))
  }
  df_pred_allna$if_n<- apply(df_pred_allna,1,if_ci)
  df_pred_allna$if_n_pan<- ifelse(df_pred_allna$if_n>0.5,1,0)
  save.image()
  ####save result
  dir.create('result')
  write.csv(df_pred_train,'result\\pred_train.csv')
  write.csv(df_pred_test,'result\\pred_test.csv')
  write.csv(df_pred_all,'result\\pred_all.csv')
  write.csv(df_pred_allna,'result\\pred_allna.csv')
  write.csv(df_roc,'result\\roc.csv')
}

###Part 2. GBRT for MBDs
library(xgboost)
library(readxl)
library(tidyverse)
library(rBayesianOptimization)
library(ROCR)

fading<-read.csv('incidence.csv') #Statutory reported incidence data
pop<-read_xlsx('code.xlsx')[,c('ADCODE99','pop_2010')] #population
collect1<-read_xlsx('collect.xlsx') #data from mosquito and host animal
#Data processing
collect1[is.na(collect1)] <- 0
data_dyn <- data.frame(ADCODE99=pop$ADCODE99,JE=0,Dengue=0)

code_yn <- unique(unlist(collect1[collect1$virus=='JEV'&(collect1$mosquito>0|(collect1$animal>0&collect1$animal_method==1)),'code']))
code_dg <- unique(unlist(collect1[collect1$virus=='DENV'&(collect1$mosquito>0||(collect1$animal>0&collect1$animal_method==1)),'code']))
data_dyn[data_dyn$ADCODE99 %in% code_yn,'JE'] <- 1
data_dyn[data_dyn$ADCODE99 %in% code_dg,'Dengue'] <- 1
#


df_rate<- pop %>% 
  left_join(fading,by=c('ADCODE99')) %>% 
  mutate(dg=if_dg/pop_2010*100000/5,nj=if_nj/pop_2010*100000/5,yn=if_yn/pop_2010*100000/5) %>% 
  left_join(data_dyn,by=c('ADCODE99'))%>% 
  mutate(shuru_dg=log(shuru_dg+0.001))

df_rate[is.na(df_rate)] <- 0
df_rate$yn_2 <- ifelse(df_rate$if_yn==0&df_rate$JE==0,0,1)
df_rate$dg_2 <- ifelse(df_rate$if_dg==0&df_rate$Dengue==0,0,1)

df_rate <- df_rate %>% dplyr::select(ADCODE99,yn_2,yn,dg_2,dg,shuru_dg) 

var <- read.csv('var_0118.csv') #The independent variables

##built model
name_all<-c('dg','yn')
for(id in 1:2){
  name<-name_all[id]
  name_l <- str_c(name,'_2')
  setwd('')
  dir.create(name)
  raw_data <- df_rate %>% left_join(var[names(var)!='shuru_dg'],by=c('ADCODE99'))
  var_name1<- read.csv('varname.csv')
  var_name <- var_name1$x[var_name1[name]==1]
  df_yn <- raw_data[c('ADCODE99',name,name_l,
                      var_name)]
  ##part.1 logistic regression
  df_yn_dg1 <- raw_data[c('ADCODE99',name,name_l,
                          var_name[!var_name %in% 'shuru_dg'])]
  params<-list(eta=0.005,max_depth=8,subsample=0.75,
               objective = "reg:logistic",eval_metric = "auc")
  df_yn1<-df_yn_dg1 %>%
    filter(get(name_l)==1)
  df_yn0<-df_yn_dg1%>%
    filter(get(name_l)==0)
  inf_t<-NULL
  #run models for 10 times with all variables to calculate relative influence
  for(i in 1:10){
    set.seed(i+1234);tt1<-sample(1:nrow(df_yn1),round(0.75*nrow(df_yn1)))
    set.seed(i+1234);tt0<-sample(1:nrow(df_yn0),round(0.75*nrow(df_yn0)))
    dtrain<-rbind(df_yn1[tt1,],df_yn0[tt0,]);dtest<-rbind(df_yn1[-tt1,],df_yn0[-tt0,])
    x_train<-xgb.DMatrix(as.matrix(dtrain[-c(1:3)]),label=as.matrix(dtrain[3]))
    x_test<-xgb.DMatrix(as.matrix(dtest[-c(1:3)]),label=as.matrix(dtest[3]))
    watchlist<-list(train=x_train,eval=x_test)
    set.seed(i+1234)
    bst_cv <- xgb.cv(params = params,data=x_train,watchlist = watchlist,verbose = 1,nfold = 5,nround = 100,
                     early_stopping_rounds = 10)
    bst<-xgb.train(params = params,data=x_train,nrounds = bst_cv$best_iteration,watchlist = watchlist,verbose = 1)
    inf<-xgb.importance(model=bst)
    if(is.null(inf_t)) inf_t<-inf[,1:2] else inf_t<-full_join(inf_t,inf[,1:2],by='Feature')
  }
  inf_t$Gain<-apply(inf_t[,2:11],1,mean,na.rm=T)
  var_inf<-inf_t$Feature[inf_t$Gain>=0.015&!is.na(inf_t$Gain)]
  #run models for 100 times with filtered variables 
  df_yn_f<-df_yn[c(name_l,var_inf)]
  x_pred<-xgb.DMatrix(as.matrix(df_yn_f[-1]),
                      label=as.matrix(df_yn_f[1]))
  df_yn1<-df_yn_f %>%
    filter(get(name_l)==1)
  df_yn0<-df_yn_f%>%
    filter(get(name_l)==0)
  
  inf_f<-NULL
  auc_train<-c();auc_test<-c()
  bst_list<-list();pred_f<-df_yn_f[name_l]
  t_all <- c();b_all <- 0
  for(i in 1:100){
    set.seed(i+1234);tt1<-sample(1:nrow(df_yn1),round(0.75*nrow(df_yn1)))
    set.seed(i+1234);tt0<-sample(1:nrow(df_yn0),round(0.75*nrow(df_yn0)))
    dtrain<-rbind(df_yn1[tt1,],df_yn0[tt0,]);dtest<-rbind(df_yn1[-tt1,],df_yn0[-tt0,])
    x_train<-xgb.DMatrix(as.matrix(dtrain[-1]),label=as.matrix(dtrain[1]))
    x_test<-xgb.DMatrix(as.matrix(dtest[-1]),label=as.matrix(dtest[1]))
    watchlist<-list(train=x_train,eval=x_test)
    set.seed(i+1234)
    bst_cv <- xgb.cv(params = params,data=x_train,watchlist = watchlist,verbose = 1,nfold = 5,nround = 100,
                     early_stopping_rounds = 10)
    
    bst<-xgb.train(params = params,data=x_train,nrounds = bst_cv$best_iteration,watchlist = watchlist,verbose = 1)
    bst_list[[i]]<-bst
    inf<-xgb.importance(model=bst)
    if(is.null(inf_f)) inf_f<-inf[,1:2] else inf_f<-full_join(inf_f,inf[,1:2],by='Feature')
    auc_train<-c(auc_train,as.numeric(bst$evaluation_log[7,2]))
    auc_test<-c(auc_test,as.numeric(bst$evaluation_log[7,3]))

    pred<-predict(bst,newdata = x_pred)
    pred_f<-cbind(pred_f,pred)

    df_roc<-prediction(pred,as.integer(pred_f[,name_l]))

    tt<-performance(df_roc,measure="sens", x.measure="spec")
    t<-mean(tt@alpha.values[[1]][(tt@y.values[[1]]+tt@x.values[[1]])==max(tt@y.values[[1]]+tt@x.values[[1]],na.rm = T)],na.rm=T)
    b <- as.numeric(pred>t)
    b_all <- b_all+b
    t_all <- c(t_all,t)
  }
  pred_f$mean<-apply(pred_f[,2:101],1,mean,na.rm=T)
  inf_f$mean<-apply(inf_f[,2:101],1,mean,na.rm=T)
  qq<-pROC::roc(as.integer(pred_f[,name_l]),pred_f$mean)

  t_mean <- mean(t_all)
  pred_f$presense<-as.integer(pred_f$mean>t_mean)
  pred_f$presense2 <- as.integer(b_all>50)
  table(pred_f$presense,as.integer(pred_f[,name_l]))
  ####Part.2 gamma regression
  params<-list(eta=0.005,max_depth=8,subsample=0.75,
               objective = "reg:gamma",
               tree_method='hist',max_bin =1500)
  df_gamma_yn<-df_yn[c(t(df_yn[name]>0)),]
  inf_t2<-NULL
  #run models for 10 times with all variables to calculate relative influence
  for(i in 1:10){
    set.seed(i+123);tt<-sample(1:nrow(df_gamma_yn),round(0.75*nrow(df_gamma_yn)))
    dtrain<-df_gamma_yn[tt,];dtest<-df_gamma_yn[-tt,]
    x_train<-xgb.DMatrix(as.matrix(dtrain[-c(1:3)]),label=as.matrix(dtrain[2]))
    x_test<-xgb.DMatrix(as.matrix(dtest[-c(1:3)]),label=as.matrix(dtest[2]))
    watchlist<-list(train=x_train,eval=x_test)
    set.seed(i+123)
    bst_cv <- xgb.cv(params = params,data=x_train,watchlist = watchlist,verbose = 1,nfold = 5,nround = 100,
                     early_stopping_rounds = 10)
    set.seed(i+123)
    bst<-xgb.train(params = params,data=x_train,nrounds = bst_cv$best_iteration,watchlist = watchlist,verbose = 1)
    inf<-xgb.importance(model=bst)
    if(is.null(inf_t2)) inf_t2<-inf[,1:2] else inf_t2<-full_join(inf_t2,inf[,1:2],by='Feature') 
  }
  inf_t2$mean<-apply(inf_t2[,c(2:11)],1,mean,na.rm=T)
  var_inf<-inf_t2$Feature[inf_t2$mean>=0.015]
  
  inf_t2[,c('Feature','mean')]
  ####run models for 100 times with filtered variables 
  df_yn_f2<-df_gamma_yn[,c(name,var_inf)]
  
  x_pred<-xgb.DMatrix(as.matrix(df_yn[var_inf]),
                      label=as.matrix(df_yn[name]))
  inf_f2<-NULL
  bst_list2<-list();pred_f2<-df_yn[name]
  loss_train<-c();loss_test<-c()
  for(i in 1:100){
    set.seed(i+1234);tt<-sample(1:nrow(df_yn_f2),round(0.75*nrow(df_yn_f2)))
    dtrain<-df_yn_f2[tt,];dtest<-df_yn_f2[-tt,]
    x_train<-xgb.DMatrix(as.matrix(dtrain[-1]),label=as.matrix(dtrain[1]))
    x_test<-xgb.DMatrix(as.matrix(dtest[-1]),label=as.matrix(dtest[1]))
    watchlist<-list(train=x_train,eval=x_test)
    set.seed(i+1234)
    bst_cv <- xgb.cv(params = params,data=x_train,watchlist = watchlist,verbose = 1,nfold = 5,nround = 100,
                     early_stopping_rounds = 10)
    set.seed(i+1234)
    bst<-xgb.train(params = params,data=x_train,nrounds =bst_cv$best_iteration,watchlist = watchlist)
    bst_list2[[i]]<-bst
    inf<-xgb.importance(model=bst)
    if(is.null(inf_f2)) inf_f2<-inf[,1:2] else inf_f2<-full_join(inf_f2,inf[,1:2],by='Feature') 
    pred<-predict(bst,newdata = x_pred)
    pred_f2<-cbind(pred_f2,pred)
    loss_train<-c(loss_train,as.numeric(bst$evaluation_log[20,2]))
    loss_test<-c(loss_test,as.numeric(bst$evaluation_log[20,3]))
  }
  inf_f2$mean<-apply(inf_f2[,c(2:101)],1,mean,na.rm=T)
  pred_f2$mean<-apply(pred_f2[,2:101],1,mean) 
  
  df_rate[,str_c(name,'_pre1')]<-pred_f$mean
  df_rate[,str_c(name,'_pre2')]<-pred_f2$mean 
  
  df_rate[,str_c(name,'_12')]<-ifelse(c(t(df_rate[,str_c(name,'_pre1')]))<=t_mean,0,
                                      # c(t(df_rate[,str_c(name,'_pre2')])))
                                      c(t(df_rate[,str_c(name,'_pre1')]))*c(t(df_rate[,str_c(name,'_pre2')])))
  b75 <- quantile(df_rate[name][df_rate[name]>0])[4]
  df_rate[,str_c(name,'_class')]<-ifelse(df_rate[name]==0&df_rate[str_c(name,'_2')]==0,1,
                                         ifelse(df_rate[name]>=b75,3,2))
  df_rate[,str_c(name,'_class_pred')]<-cut(c(t(df_rate[,str_c(name,'_12')])),
                                           breaks=c(-1,0,
                                                    quantile(df_rate[,name][df_rate[,name]>0])[c(4,5)]),
                                           labels=c(1,2,3))
  
  table(df_rate[,c(str_c(name,'_class'),str_c(name,'_class_pred'))])
  write.csv(pred_f2,paste0(name,'\\pred_gamma.csv'),row.names=F)
  write.csv(inf_f2,paste0(name,'\\inf_gamma.csv'),row.names=F)
  write.csv(pred_f,paste0(name,'\\pred.csv'),row.names=F)
  write.csv(inf_f,paste0(name,'\\inf.csv'),row.names=F)
  save.image(file = paste0(name,'\\xgb_gamma.Rdata'))
}





